from tornado.web import RequestHandler, HTTPError
from tornado.httpclient import AsyncHTTPClient
from base.django_handler_mixin import DjangoHandlerMixin

ALLOWED_DOMAINS = {
    'search.gesis.org': True,
    'api.datacite.org': True
}


class Proxy(DjangoHandlerMixin, RequestHandler):
    async def get(self, url):
        user = self.get_current_user()
        domain = url.split('/')[2]
        if domain not in ALLOWED_DOMAINS or not user.is_authenticated:
            self.set_status(401)
            self.finish()
            return
        query = self.request.query
        if query:
            url += '?' + query
        http = AsyncHTTPClient()
        response = await http.fetch(
            url,
            method='GET'
        )
        if not response.error:
            self.write(response.body)
        self.finish()



from avatar.utils import get_primary_avatar, get_default_avatar_url


def string_to_color(username):
    hash = 0
    for ch in username:
        hash = ord(ch) + ((hash << 5) - hash)

    r = str((hash >> (0 * 8)) & 255)
    g = str((hash >> (1 * 8)) & 255)
    b = str((hash >> (2 * 8)) & 255)

    return 'rgb(' + r + ',' + g + ',' + b + ')'


def get_user_avatar_url(user):
    avatar = get_primary_avatar(user, 80)
    if avatar:
        url = avatar.avatar_url(80)
        return {
            'url': url,
            'uploaded': True,
            'html': (
                '<img class="fw-avatar" src="' +
                url +
                '" alt="' + user.username + '">'
            )
        }
    else:
        cl = string_to_color(user.username)
        return {
            'url': get_default_avatar_url(),
            'uploaded': False,
            'html': (
                '<span class="fw-string-avatar" style="background-color: ' +
                cl +
                ';"><span>' +
                user.username[0] +
                '</span></span>'
            )
        }



# Generated by Django 2.1.4 on 2019-01-18 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0005_document_template'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.0, max_digits=3),
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
from django.core.mail import send_mail
from django.conf import settings


class Feedback(models.Model):
    message = models.TextField()
    owner = models.ForeignKey(
        User,
        blank=True,
        null=True,
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        if self.owner:
            return self.owner.username + ': ' + self.message
        else:
            return 'Anonymous: ' + self.message

    def save(self, *args, **kwargs):
        reply_to = ''
        if self.owner:
            from_sender = self.owner.username
            if self.owner.email:
                reply_to = ' (' + self.owner.email + ')'
        else:
            from_sender = 'Anonymous'
        send_mail('Feedback from ' + from_sender + reply_to,
                  self.message,
                  settings.DEFAULT_FROM_EMAIL,
                  [settings.CONTACT_EMAIL],
                  fail_silently=True)
        super(Feedback, self).save(*args, **kwargs)



from datetime import datetime, timedelta

from django.conf import settings

from tornado.web import RequestHandler, StaticFileHandler, HTTPError


class HelloHandler(RequestHandler):

    def head(self):
        self.finish()

    def get(self):
        self.write('Hello from tornado')


class RobotsHandler(RequestHandler):

    def head(self):
        self.finish()

    def get(self):
        self.write(
            (
                'User-agent: *\nDisallow: /*\nAllow: /$'
            )
        )


class DjangoStaticFilesHandler(StaticFileHandler):

    def initialize(self, default_filename=None):
        super(DjangoStaticFilesHandler, self).initialize(None,
                                                         default_filename=None)

    def validate_absolute_path(self, root, absolute_path):
        if absolute_path is False:
            raise HTTPError(404)
        return absolute_path

    def get_absolute_path(self, root, path):
        for finder in settings.STATICFILES_FINDERS:
            finderClass = self.get_class(finder)
            instance = finderClass()

            new_path = instance.find(path)
            if len(new_path) > 0:
                return new_path
        return False

    def get_class(self, kls):
        parts = kls.split('.')
        module = '.'.join(parts[:-1])
        m = __import__(module)
        for comp in parts[1:]:
            m = getattr(m, comp)
        return m

    def set_extra_headers(self, path):
        if settings.DEBUG:
            self.set_header('Cache-Control', 'no-cache, must-revalidate')
            self.set_header('Expires', '0')
            expiration = datetime.now() - timedelta(days=366)
            self.set_header('Last-Modified', expiration)
        else:
            self.set_header(
                'Expires',
                datetime.utcnow() + timedelta(days=365*10)
            )
            self.set_header('Cache-Control', "max-age=" + str(86400*365*10))
            self.set_header('Last-Modified', datetime.utcnow())



import time

from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains

from allauth.account.models import EmailConfirmationHMAC, EmailAddress


class EditorTest(LiveTornadoTestCase, SeleniumHelper):
    fixtures = [
        'initial_documenttemplates.json',
        'initial_styles.json',
    ]

    @classmethod
    def setUpClass(cls):
        super(EditorTest, cls).setUpClass()
        cls.base_url = cls.live_server_url
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super(EditorTest, cls).tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user1 = self.create_user(
            username='Yeti',
            email='yeti@snowman.com',
            passtext='otter'
        )

    def tearDown(self):
        self.leave_site(self.driver)

    def test_track_changes(self):
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".new_document button"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "A test article with tracked changes"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "First I type "
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "some"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            " standard "
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "text"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            " here.\nI'll even write a second paragraph."
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(5) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        ActionChains(self.driver).double_click(
            self.driver.find_element(By.CSS_SELECTOR, ".article-body strong")
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        ActionChains(self.driver).double_click(
            self.driver.find_element(By.CSS_SELECTOR, ".article-body em")
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "button[title=Strong]"
        ).click()
        ActionChains(self.driver).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.BACKSPACE
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            "insertion"
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.RIGHT
        ).send_keys(
            Keys.ENTER
        ).send_keys(
            Keys.UP
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".editor-toolbar .multiButtons"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".editor-toolbar .multiButtons .ui-button:nth-child(5)"
        ).click()

        change_tracking_boxes = self.driver.find_elements_by_css_selector(
            '.margin-box.track'
        )
        self.assertEqual(
            len(change_tracking_boxes),
            6
        )

    def test_share_document(self):
        self.create_user(
            username='Yeti2',
            email='yeti2@snowman.com',
            passtext='otter'
        )
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".new_document button"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "A test article to share"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".header-menu:nth-child(1) > .header-nav-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "li:nth-child(1) > .fw-pulldown-item"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti2@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-add-button"
        ).click()
        self.driver.find_element(
            By.ID,
            "new-member-user-string"
        ).click()
        self.driver.find_element(By.ID, "new-member-user-string").send_keys(
            "yeti3@snowman.com"
        )
        ActionChains(self.driver).send_keys(
            Keys.TAB
        ).send_keys(
            Keys.RETURN
        ).perform()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".collaborator-tr .fa-caret-down"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".fw-pulldown-item[data-rights=write]"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".ui-dialog .fw-dark"
        ).click()
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        self.driver.find_element(
            By.ID,
            "preferences-btn"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".fw-logout-button"
        ).click()
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti2")
        self.driver.find_element(By.ID, "id_password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr'
        )
        self.assertEqual(
            len(documents),
            1
        )
        write_access_rights = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr .icon-access-write'
        )
        self.assertEqual(
            len(write_access_rights),
            1
        )
        self.driver.find_element_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "The body"
        )
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-title"
        ).text == "A test article to share"
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body"
        ).text == "The body"
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        self.driver.find_element(
            By.ID,
            "preferences-btn"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".fw-logout-button"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[title="Sign up"]'
        ).click()
        self.driver.find_element(
            By.ID,
            'id_username'
        ).send_keys('Yeti3')
        self.driver.find_element(
            By.ID,
            'id_password1'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_password2'
        ).send_keys('password')
        self.driver.find_element(
            By.ID,
            'id_email'
        ).send_keys('yeti3@snowman.com')
        self.driver.find_element(
            By.ID,
            'signup-submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    'a[href="mailto:yeti3@snowman.com"]'
                )
            )
        )
        email = EmailAddress.objects.filter(email='yeti3@snowman.com').first()
        self.assertIsNotNone(email)
        confirmation_key = EmailConfirmationHMAC(email).key
        self.driver.get(
            self.base_url +
            "/account/confirm-email/" +
            confirmation_key +
            "/"
        )
        self.driver.find_element(
            By.ID,
            'terms-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'test-check'
        ).click()
        self.driver.find_element(
            By.ID,
            'submit'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.text_to_be_present_in_element(
                (
                    By.CSS_SELECTOR,
                    ".fw-contents h1"
                ),
                "Thanks for verifying!"
            )
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            'a[href="/"]'
        ).click()
        self.driver.find_element(By.ID, "id_login").send_keys("Yeti3")
        self.driver.find_element(By.ID, "id_password").send_keys("password")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".new_document button"
                )
            )
        )
        documents = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr'
        )
        self.assertEqual(
            len(documents),
            1
        )
        read_access_rights = self.driver.find_elements_by_css_selector(
            '.fw-contents tbody tr .icon-access-read'
        )
        self.assertEqual(
            len(read_access_rights),
            1
        )
        self.driver.find_element_by_css_selector(
            '.fw-contents tbody tr a.doc-title'
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "Some extra content that doesn't show"
        )
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-title"
        ).text == "A test article to share"
        time.sleep(0.5)  # Wait for prosemirror to synchronize
        assert self.driver.find_element(
            By.CSS_SELECTOR,
            ".article-body"
        ).text == "The body"
        self.driver.find_element(
            By.ID,
            "close-document-top"
        ).click()
        self.driver.find_element(
            By.ID,
            "preferences-btn"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            ".fw-logout-button"
        ).click()



from django.core.management.base import BaseCommand
from django.core.management import call_command
from django.core.management.base import CommandError


class Command(BaseCommand):
    help = 'Lint JavaScript and Python files'

    def handle(self, *args, **options):
        try:
            call_command("lint_js")
        except CommandError:
            pass
        call_command("lint_py")



from django.core.management.base import BaseCommand
import zipfile
import os
import magic
from subprocess import call

from django.conf import settings


def zip_folder(path, zip_file):
    file_paths = []
    for root, dirs, files in os.walk(path):
        for file in files:
            out_file = os.path.join(root, file)
            relative_out_file = os.path.relpath(out_file, path)
            zip_file.write(out_file, relative_out_file)
            mimetype = magic.from_file(out_file, mime=True)
            # Override mimetype for CSS files
            if out_file.split('.')[-1].lower() == 'css':
                mimetype = 'text/css'
            file_paths.append({
                'path': relative_out_file,
                'mimetype': mimetype
            })
    return file_paths


def opf_entries(file_paths):
    opf_text = (
        '// This file is auto-generated. CHANGES WILL BE OVERWRITTEN! '
        'Re-generate by running ./manage.py bundle_mathlive.\n'
    )
    opf_text += 'export const mathliveOpfIncludes = `\n'
    for index, file_path in enumerate(file_paths):
        opf_text += '<item id="mathlive-%d" href="%s" media-type="%s" />\n' % (
            index, file_path['path'], file_path['mimetype'])
    opf_text += '`'
    return opf_text


class Command(BaseCommand):
    args = ''
    help = (
        'Create a zip file containing the mathlive style files to be bundled '
        'with EPUB and HTML exports'
    )

    def handle(self, *args, **options):
        print("Bundling MathLive")
        # Copy MathLive CSS
        mathlive_css_path = os.path.join(
            settings.PROJECT_PATH, "static-libs/css/libs/mathlive/")
        if not os.path.exists(mathlive_css_path):
            os.makedirs(mathlive_css_path)
        call([
            "cp",
            "--preserve=mode",
            os.path.join(
                settings.PROJECT_PATH,
                ".transpile/node_modules/mathlive/dist/mathlive.css"
            ),
            mathlive_css_path
        ])
        call([
            "cp",
            "-r",
            "--preserve=mode",
            os.path.join(
                settings.PROJECT_PATH,
                ".transpile/node_modules/mathlive/dist/fonts"
            ),
            mathlive_css_path
        ])
        zip_file_path = os.path.join(
            settings.PROJECT_PATH, 'static-libs/zip/mathlive_style.zip')
        zip_dir = os.path.dirname(zip_file_path)
        if not os.path.exists(zip_dir):
            os.makedirs(zip_dir)
        if os.path.exists(zip_file_path):
            os.remove(zip_file_path)
        zip_file = zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED)
        in_dir = os.path.dirname(mathlive_css_path)
        file_paths = zip_folder(in_dir, zip_file)
        zip_file.close()

        opf_file_contents = opf_entries(file_paths)
        opf_file_path = os.path.join(
            settings.PROJECT_PATH,
            'static-libs/js/modules/mathlive/opf_includes.js'
        )
        opf_dir = os.path.dirname(opf_file_path)
        if not os.path.exists(opf_dir):
            os.makedirs(opf_dir)
        opf_file = open(opf_file_path, "wb")
        opf_file.write(opf_file_contents.encode('utf8'))
        opf_file.close()



from django.apps import AppConfig
from django.core.management import call_command

from npm_mjs.signals import post_npm_install


def bundle_mathlive(sender, **kwargs):
    call_command("bundle_mathlive")


class BaseConfig(AppConfig):
    name = 'base'

    def ready(self):
        post_npm_install.connect(bundle_mathlive)



import json

from django.http import JsonResponse, HttpResponseRedirect
from django.contrib.auth import logout, update_session_auth_hash
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User

from .forms import UserForm, TeamMemberForm
from . import util as userutil
from document.models import AccessRight, AccessRightInvite

from allauth.account.models import (
    EmailAddress,
    EmailConfirmation,
    EmailConfirmationHMAC
)
from allauth.account.views import SignupView
from allauth.account import signals
from django.contrib.auth.forms import PasswordChangeForm
from allauth.account.forms import AddEmailForm

from avatar.models import Avatar
from avatar import views as avatarviews
from avatar.forms import UploadAvatarForm
from avatar.signals import avatar_updated

from document.views import apply_invite


def logout_page(request):
    """
    Log users out and re-direct them to the main page.
    """
    logout(request)
    return HttpResponseRedirect('/')


def info(request):
    """
    Get user profile info
    """
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse({}, status=405)
    if request.user.is_authenticated:
        response = {
            'id': request.user.id,
            'username': request.user.username,
            'first_name': request.user.first_name,
            'name': request.user.readable_name,
            'last_name': request.user.last_name,
            'avatar': userutil.get_user_avatar_url(request.user),
            'emails': [],
            'is_authenticated': True
        }

        for emailaddress in request.user.emailaddress_set.all():
            email = {
                'address': emailaddress.email,
            }
            if emailaddress.primary:
                email['primary'] = True
            if emailaddress.verified:
                email['verified'] = True
            response['emails'].append(email)
    else:
        response = {
            'is_authenticated': False
        }
    return JsonResponse(
        response,
        status=200
    )


@login_required
def password_change(request):
    '''
    Change password
    '''
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        form = PasswordChangeForm(user=request.user, data=request.POST)
        if form.is_valid():
            status = 200
            form.save()
            # Updating the password logs out all other sessions for the user
            # except the current one.
            update_session_auth_hash(request, form.user)
        else:
            response['msg'] = form.errors
            status = 201

    return JsonResponse(
        response,
        status=status
    )


@login_required
def add_email(request):
    '''
    Add email address
    '''
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        add_email_form = AddEmailForm(request.user, request.POST)
        if add_email_form.is_valid():
            status = 200
            email_address = add_email_form.save(request)
            signals.email_added.send(
                sender=request.user.__class__,
                request=request, user=request.user,
                email_address=email_address
            )
        else:
            status = 201
            response['msg'] = add_email_form.errors

    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete_email(request):
    response = {}
    status = 405
    email = request.POST["email"]
    if request.is_ajax() and request.method == 'POST':
        response['msg'] = "Removed e-mail address " + email
        status = 200
        try:
            email_address = EmailAddress.objects.get(
                user=request.user,
                email=email
            )
            if email_address.primary:
                status = 201
                msg = "You cannot remove your primary e-mail address " + email
                response['msg'] = msg

            else:
                email_address.delete()
                signals.email_removed.send(
                    sender=request.user.__class__,
                    request=request,
                    user=request.user,
                    email_address=email_address
                )

        except EmailAddress.DoesNotExist:
            pass

    return JsonResponse(
        response,
        status=status
    )


@login_required
def primary_email(request):
    response = {}
    status = 405
    email = request.POST["email"]
    if request.is_ajax() and request.method == 'POST':
        try:
            email_address = EmailAddress.objects.get(
                user=request.user,
                email=email,
            )
            if not email_address.verified:
                status = 201
                msg = "Your primary e-mail address must be verified"
                response['msg'] = msg
            else:
                # Sending the old primary address to the signal
                # adds a db query.
                try:
                    from_email_address = EmailAddress.objects.get(
                        user=request.user,
                        primary=True
                    )
                except EmailAddress.DoesNotExist:
                    from_email_address = None

                status = 200
                email_address.set_as_primary()
                response['msg'] = "Primary e-mail address set"
                signals.email_changed.send(
                    sender=request.user.__class__,
                    request=request, user=request.user,
                    from_email_address=from_email_address,
                    to_email_address=email_address
                )
        except EmailAddress.DoesNotExist:
            status = 201
            response['msg'] = "e-mail address does not exist"

    return JsonResponse(
        response,
        status=status
    )


@login_required
def upload_avatar(request):
    '''
    Upload avatar image
    '''
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':

        avatar, avatars = avatarviews._get_avatars(request.user)
        upload_avatar_form = UploadAvatarForm(
            None,
            request.FILES,
            user=request.user
        )
        if upload_avatar_form.is_valid():
            avatar = Avatar(
                user=request.user,
                primary=True,
            )
            image_file = request.FILES['avatar']
            avatar.avatar.save(image_file.name, image_file)
            avatar.save()
            avatar_updated.send(
                sender=Avatar,
                user=request.user,
                avatar=avatar
            )
            response['avatar'] = userutil.get_user_avatar_url(
                request.user
            )['url']
            status = 200
    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete_avatar(request):
    '''
    Delete avatar image
    '''
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        avatar, avatars = avatarviews._get_avatars(request.user)
        if avatar is None:
            response['error'] = 'User has no avatar'
        else:
            aid = avatar.id
            for a in avatars:
                if a.id == aid:
                    a.primary = True
                    a.save()
                    avatar_updated.send(
                        sender=Avatar,
                        user=request.user,
                        avatar=avatar
                    )
                    break
            Avatar.objects.filter(pk=aid).delete()
            response['avatar'] = userutil.get_user_avatar_url(
                request.user
            )['url']
            status = 200
    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete_user(request):
    """
    Delete the user
    """
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        user = request.user
        # Only remove users who are not marked as having staff status
        # to prevent administratoras from deleting themselves accidentally.
        if not user.check_password(request.POST['password']):
            status = 401
        elif user.is_staff:
            status = 403
        else:
            logout(request)
            user.delete()
            status = 204
    return JsonResponse(
        response,
        status=status
    )


@login_required
def save_profile(request):
    """
    Save user profile information
    """
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        form_data = json.loads(request.POST['form_data'])
        user_object = User.objects.get(pk=request.user.pk)
        user_form = UserForm(form_data['user'], instance=user_object)
        if user_form.is_valid():
            user_form.save()
            status = 200
        else:
            response['errors'] = user_form.errors
            status = 422
        '''
        currently not used
        profile_object = user_object.profile
        profile_form = UserProfileForm(
            form_data['profile'],
            instance=profile_object
        )
        if profile_form.is_valid():
            if status == 200:
                user_form.save()
                profile_form.save()
        else:
            if status == 200:
                response['errors']=profile_form.errors
                status = 422
            else:
                response['errors']+=profile_form.errors
        '''

    return JsonResponse(
        response,
        status=status
    )


@login_required
def list_team_members(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        response['team_members'] = []

        for member in User.objects.filter(member__leader=request.user):
            team_member = {
                'id': member.id,
                'name': member.readable_name,
                'username': member.get_username(),
                'email': member.email,
                'avatar': userutil.get_user_avatar_url(member)
            }
            response['team_members'].append(team_member)
    return JsonResponse(
        response,
        status=status
    )


@login_required
def add_team_member(request):
    """
    Add a user as a team member of the current user
    """
    response = {}
    status = 405
    new_member = False
    if request.is_ajax() and request.method == 'POST':
        status = 202
        user_string = request.POST['user_string']
        if "@" in user_string and "." in user_string:
            email_address = EmailAddress.objects.filter(
                email=user_string
            ).first()
            if email_address:
                new_member = email_address.user
        else:
            user = User.objects.filter(username=user_string).first()
            if user:
                new_member = user
        if new_member:
            if new_member.pk is request.user.pk:
                # 'You cannot add yourself to your contacts!'
                response['error'] = 1
            else:
                form_data = {
                    'leader': request.user.pk,
                    'member': new_member.pk
                }
                team_member_form = TeamMemberForm(form_data)
                if team_member_form.is_valid():
                    team_member_form.save()
                    the_avatar = userutil.get_user_avatar_url(new_member)
                    response['member'] = {
                        'id': new_member.pk,
                        'name': new_member.username,
                        'email': new_member.email,
                        'avatar': the_avatar
                    }
                    status = 201
                else:
                    # 'This person is already in your contacts!'
                    response['error'] = 2
        else:
            # 'User cannot be found'
            response['error'] = 3

    return JsonResponse(
        response,
        status=status
    )


@login_required
def change_team_member_roles(request):
    """
    Change the roles of a team member
    """
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        form_data = json.loads(request.POST['form_data'])
        form_data['leader'] = request.user.pk
        member = User.objects.get(pk=form_data['member'])
        team_member_object_instance = request.user.leader.filter(
            member=member
        ).first()
        team_member_form = TeamMemberForm(
            form_data,
            instance=team_member_object_instance
        )
        if team_member_form.is_valid():
            team_member_form.save()
            status = 200
    return JsonResponse(
        response,
        status=status
    )


@login_required
def remove_team_member(request):
    """
    Remove a team member
    """
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        former_members = request.POST.getlist('members[]')
        for former_member in former_members:
            former_member = int(former_member)
            # Revoke all permissions given to this person
            AccessRight.objects.filter(
                user_id=former_member,
                document__owner=request.user
            ).delete()
            # Now delete the user from the team
            team_member_object_instance = request.user.leader.filter(
                member_id=former_member
            ).first()
            team_member_object_instance.delete()
        status = 200
    return JsonResponse(
        response,
        status=status
    )


def get_confirmkey_data(request):
    """
    Get data for an email confirmation key
    """
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        key = request.POST['key']
        confirmation = EmailConfirmationHMAC.from_key(key)
        if not confirmation:
            qs = EmailConfirmation.objects.all_valid()
            qs = qs.select_related("email_address__user")
            confirmation = qs.filter(key=key.lower()).first()
        if confirmation:
            status = 200
            response['username'] = confirmation.email_address.user.username
            response['email'] = confirmation.email_address.email
        else:
            status = 404
    return JsonResponse(
        response,
        status=status
    )


class FidusSignupView(SignupView):
    def form_valid(self, form):
        ret = super(FidusSignupView, self).form_valid(form)
        if ret.status_code > 399:
            return ret
        if 'invite_id' in self.request.POST:
            invite_id = int(self.request.POST['invite_id'])
            inv = AccessRightInvite.objects.filter(id=invite_id).first()
            if inv:
                apply_invite(inv, self.user)
        else:
            inv = AccessRightInvite.objects.filter(
                email=self.user.email
            ).first()
            if inv:
                apply_invite(inv, self.user)
        return ret


signup = FidusSignupView.as_view()



# Generated by Django 1.11.13 on 2018-08-14 17:32
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('bibliography', '0001_initial'), ('bibliography', '0002_entry_bib_type'), ('bibliography', '0003_auto_20161113_1534'), ('bibliography', '0004_remove_entry_entry_type'), ('bibliography', '0005_auto_20161115_0310'), ('bibliography', '0006_auto_20161122_0304'), ('bibliography', '0007_auto_20161201_0209'), ('bibliography', '0008_auto_20161201_0545'), ('bibliography', '0009_auto_20161207_0130'), ('bibliography', '0010_auto_20161207_0133'), ('bibliography', '0011_auto_20170101_1647')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('entry_key', models.CharField(max_length=64)),
                ('entry_cat', models.CharField(default='', max_length=255)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('fields', models.TextField(default='{}')),
                ('entry_owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('bib_type', models.CharField(default='', max_length=30)),
            ],
        ),
        migrations.CreateModel(
            name='EntryCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category_title', models.CharField(max_length=100)),
                ('category_owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Entry categories',
            },
        ),
        migrations.AlterUniqueTogether(
            name='entry',
            unique_together=set([]),
        ),
        migrations.AlterField(
            model_name='entry',
            name='entry_cat',
            field=models.TextField(default='[]'),
        ),
    ]



from selenium.common.exceptions import NoSuchElementException
import time

from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper


class EditProfileTest(LiveTornadoTestCase, SeleniumHelper):

    @classmethod
    def setUpClass(cls):
        super(EditProfileTest, cls).setUpClass()
        cls.base_url = cls.live_server_url
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super(EditProfileTest, cls).tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user = self.create_user(
            username='Yeti',
            email='yeti@snowman.com',
            passtext='otter1'
        )
        self.login_user(self.user, self.driver, self.client)

    def test_edit_profile(self):
        driver = self.driver
        driver.get(self.base_url + "/")
        driver.find_element_by_id("preferences-btn").click()
        driver.find_element_by_css_selector(".fw-avatar-card").click()
        driver.find_element_by_id("first_name").clear()
        driver.find_element_by_id("first_name").send_keys("Snowman")
        driver.find_element_by_id("last_name").clear()
        driver.find_element_by_id("last_name").send_keys("Yeti")
        driver.find_element_by_id("submit-profile").click()
        time.sleep(0.3)
        for i in range(100):
            if len(
                driver.find_elements_by_css_selector(
                    "#fw-edit-profile-pwd"
                )
            ) == 1:
                break
            time.sleep(0.1)
        else:
            self.fail("time out")
        driver.find_element_by_id("fw-edit-profile-pwd").click()
        driver.find_element_by_id("old-password-input").clear()
        driver.find_element_by_id("old-password-input").send_keys("otter1")
        driver.find_element_by_id("new-password-input1").clear()
        driver.find_element_by_id("new-password-input1").send_keys("otter2")
        driver.find_element_by_id("new-password-input2").clear()
        driver.find_element_by_id("new-password-input2").send_keys("otter2")
        driver.find_element_by_xpath("(//button[@type='button'])[2]").click()
        for i in range(100):
            time.sleep(0.1)
            if len(
                driver.find_elements_by_css_selector(
                    "#alerts-wrapper .alerts-info"
                )
            ) > 0:
                break
        else:
            self.fail("time out")
        driver.find_element_by_id("fw-edit-profile-pwd").click()
        driver.find_element_by_id("old-password-input").clear()
        driver.find_element_by_id("old-password-input").send_keys("otter2")
        driver.find_element_by_id("new-password-input1").clear()
        driver.find_element_by_id("new-password-input1").send_keys("otter1")
        driver.find_element_by_id("new-password-input2").clear()
        driver.find_element_by_id("new-password-input2").send_keys("otter1")
        driver.find_element_by_xpath("(//button[@type='button'])[2]").click()
        for i in range(30):
            time.sleep(0.3)
            if len(
                driver.find_elements_by_css_selector("#fw-change-pwd-dialog")
            ) == 0:
                break
        else:
            self.fail("time out")
        driver.refresh()
        try:
            self.assertEqual("Yeti", driver.find_element_by_id(
                "last_name").get_attribute("value"))
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        try:
            self.assertEqual("Snowman", driver.find_element_by_id(
                "first_name").get_attribute("value"))
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        try:
            self.assertEqual("Yeti", driver.find_element_by_id(
                "username").get_attribute("value"))
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        driver.find_element_by_id("first_name").clear()
        driver.find_element_by_id("first_name").send_keys("")
        driver.find_element_by_id("last_name").clear()
        driver.find_element_by_id("last_name").send_keys("")
        driver.find_element_by_id("submit-profile").click()
        time.sleep(1)
        for i in range(30):
            time.sleep(0.3)
            if len(
                driver.find_elements_by_css_selector(
                    "#preferences-btn"
                )
            ) == 1:
                break
        else:
            self.fail("time out")
        driver.find_element_by_id("preferences-btn").click()
        driver.find_element_by_css_selector("button.fw-logout-button").click()
        time.sleep(1)
        try:
            self.assertEqual("Login - Fidus Writer", driver.title)
        except AssertionError as e:
            self.verificationErrors.append(str(e))

    def is_element_present(self, how, what):
        try:
            self.driver.find_element(by=how, value=what)
        except NoSuchElementException:
            return False
        return True

    def tearDown(self):
        self.assertEqual([], self.verificationErrors)



from builtins import object
from django.forms import ModelForm

from django.contrib.auth.models import User
from .models import UserProfile, TeamMember


class UserForm(ModelForm):
    class Meta(object):
        model = User
        fields = ('username', 'first_name', 'last_name')


class UserProfileForm(ModelForm):
    class Meta(object):
        model = UserProfile
        fields = ('about',)


class TeamMemberForm(ModelForm):
    class Meta(object):
        model = TeamMember
        exclude = []



# Generated by Django 1.11.13 on 2018-08-14 17:09
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import document.models


class Migration(migrations.Migration):

    replaces = [('document', '0001_initial'), ('document', '0002_auto_20151230_1732'), ('document', '0003_auto_20160104_0329'), ('document', '0004_auto_20160104_0512'), ('document', '0005_auto_20160119_0716'), ('document', '0006_auto_20160130_0708'), ('document', '0007_auto_20160130_0709'), ('document', '0008_auto_20160515_0838'), ('document', '0009_auto_20160515_0847'), ('document', '0010_auto_20160515_0849'), ('document', '0011_auto_20160515_0925'), ('document', '0012_exporttemplate'), ('document', '0013_auto_20161003_1756'), ('document', '0014_auto_20161008_0305'), ('document', '0015_auto_20161018_0531'), ('document', '0016_auto_20161031_1616'), ('document', '0017_submission'), ('document', '0018_submission_user'), ('document', '0019_auto_20161212_0657'), ('document', '0020_submittedaccessright_submission_id'), ('document', '0021_auto_20161212_0746'), ('document', '0017_auto_20161217_0320'), ('document', '0022_merge_20161219_0605'), ('document', '0023_auto_20170122_0525'), ('document', '0024_auto_20170604_1811'), ('document', '0025_auto_20170612_1123'), ('document', '0026_auto_20170810_0956'), ('document', '0027_auto_20170810_0957'), ('document', '0028_remove_document_settings'), ('document', '0029_auto_20170810_1130'), ('document', '0030_document_bibliography'), ('document', '0031_auto_20180324_1935'), ('document', '0032_document_listed')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AccessRight',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rights', models.CharField(choices=[('r', 'read'), ('w', 'read/write')], max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='', max_length=255)),
                ('contents', models.TextField(default='{}')),
                ('metadata', models.TextField(default='{}')),
                ('added', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='owner', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='DocumentRevision',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('note', models.CharField(blank=True, default='', max_length=255)),
                ('date', models.DateTimeField(auto_now=True)),
                ('file_object', models.FileField(upload_to=document.models.revision_filename)),
                ('file_name', models.CharField(blank=True, default='', max_length=255)),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.Document')),
            ],
        ),
        migrations.AddField(
            model_name='accessright',
            name='document',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.Document'),
        ),
        migrations.AddField(
            model_name='accessright',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('r', 'reader'), ('w', 'author'), ('e', 'editor'), ('c', 'reviewer')], max_length=3),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('r', 'reader'), ('w', 'author'), ('e', 'editor'), ('c', 'reviewer')], max_length=1),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('r', 'reader'), ('w', 'author'), ('e', 'editor'), ('c', 'reviewer'), ('o', 'comment_only')], max_length=1),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('r', 'reader'), ('w', 'author'), ('e', 'editor'), ('c', 'reviewer'), ('o', 'comment_only')], max_length=7),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('read', 'Reader'), ('write', 'Writer'), ('edit', 'Editor'), ('review', 'Reviewer'), ('comment', 'Commentator')], max_length=7),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('read', 'Reader'), ('read-without-comments', 'Read without comments'), ('write', 'Writer'), ('review', 'Reviewer'), ('comment', 'Commentator'), ('edit', 'Editor')], max_length=21),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('read', 'Reader'), ('read-without-comments', 'Reader without comment access'), ('write', 'Writer'), ('review', 'Reviewer'), ('comment', 'Commentator'), ('edit', 'Editor')], max_length=21),
        ),
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('read', 'Reader'), ('read-without-comments', 'Reader without comment access'), ('write', 'Writer'), ('write-tracked', 'Write with tracked changes'), ('review', 'Reviewer'), ('comment', 'Commentator'), ('edit', 'Editor')], max_length=21),
        ),
        migrations.AlterUniqueTogether(
            name='accessright',
            unique_together=set([('document', 'user')]),
        ),
        migrations.AddField(
            model_name='document',
            name='diff_version',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='document',
            name='last_diffs',
            field=models.TextField(default='[]'),
        ),
        migrations.AddField(
            model_name='document',
            name='version',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='document',
            name='comment_version',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='document',
            name='comments',
            field=models.TextField(default='{}'),
        ),
        migrations.CreateModel(
            name='ExportTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_name', models.CharField(blank=True, default='', max_length=255)),
                ('file_type', models.CharField(choices=[('docx', 'Docx'), ('odt', 'ODT')], max_length=5)),
                ('template_file', models.FileField(upload_to='export_template')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='exporttemplate',
            unique_together=set([('file_name', 'file_type')]),
        ),
        migrations.RemoveField(
            model_name='document',
            name='comment_version',
        ),
        migrations.RemoveField(
            model_name='document',
            name='diff_version',
        ),
        migrations.RemoveField(
            model_name='document',
            name='metadata',
        ),
        migrations.AddField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=2.0, max_digits=3),
        ),
        migrations.AlterModelOptions(
            name='document',
            options={'ordering': ['-id']},
        ),
        migrations.AddField(
            model_name='document',
            name='bibliography',
            field=models.TextField(default='{}'),
        ),
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=2.1, max_digits=3),
        ),
        migrations.AddField(
            model_name='document',
            name='listed',
            field=models.BooleanField(default=True),
        ),
    ]



from builtins import map
from builtins import filter
from time import mktime

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.core.serializers.python import Serializer
from django.utils.translation import ugettext as _

from usermedia.models import Image, ImageCategory, UserImage
from .models import ALLOWED_FILETYPES


class SimpleSerializer(Serializer):

    def end_object(self, obj):
        self._current['id'] = obj._get_pk_val()
        self.objects.append(self._current)


serializer = SimpleSerializer()


# save changes or create a new entry
@login_required
def save(request):
    response = {}
    response['errormsg'] = {}
    status = 403
    if request.is_ajax() and request.method == 'POST':
        if 'image' in request.FILES and \
                request.FILES['image'].content_type not in ALLOWED_FILETYPES:
            status = 200  # Not implemented
            response['errormsg']['error'] = _('Filetype not supported')
        else:
            image = False
            if 'id' in request.POST and 'image' not in request.FILES:
                user_image = UserImage.objects.filter(
                    image_id=int(request.POST['id']),
                    owner=request.user
                ).first()
                if user_image:
                    image = user_image.image
                    status = 200
            if image is False:
                image = Image()
                image.uploader = request.user
                user_image = UserImage()
                user_image.owner = request.user
                status = 201
                if 'checksum' in request.POST:
                    image.checksum = request.POST['checksum']
            user_image.title = request.POST['title']
            if 'cats' in request.POST:
                user_image.image_cat = request.POST['cats']
            if 'image' in request.FILES:
                image.image = request.FILES['image']
            if status == 201 and 'image' not in request.FILES:
                status = 200
                response['errormsg']['error'] = _('No file uploaded')
            else:
                image.save()
                user_image.image = image
                user_image.save()
                response['values'] = {
                    'id': image.id,
                    'title': user_image.title,
                    'image': image.image.url,
                    'file_type': image.file_type,
                    'added': mktime(image.added.timetuple()) * 1000,
                    'checksum': image.checksum,
                    'cats': list(
                        map(
                            int,
                            list(filter(bool, user_image.image_cat.split(',')))
                        )
                    )
                }
                if image.thumbnail:
                    response['values']['thumbnail'] = image.thumbnail.url
                    response['values']['height'] = image.height
                    response['values']['width'] = image.width
    return JsonResponse(
        response,
        status=status
    )


# delete an image
@login_required
def delete(request):
    status = 405
    response = {}
    if request.is_ajax() and request.method == 'POST':
        status = 201
        ids = request.POST.getlist('ids[]')
        UserImage.objects.filter(
            image_id__in=ids,
            owner=request.user
        ).delete()
        for image in Image.objects.filter(id__in=ids):
            if image.is_deletable():
                image.delete()
    return JsonResponse(
        response,
        status=status
    )


# returns list of images
@login_required
def images(request):
    response = {}
    status = 403
    if request.is_ajax() and request.method == 'POST':
        status = 200
        response['imageCategories'] = serializer.serialize(
            ImageCategory.objects.filter(category_owner=request.user))
        response['images'] = []
        user_images = UserImage.objects.filter(owner=request.user)
        for user_image in user_images:
            image = user_image.image
            if image.image:
                field_obj = {
                    'id': image.id,
                    'title': user_image.title,
                    'image': image.image.url,
                    'file_type': image.file_type,
                    'added': mktime(image.added.timetuple()) * 1000,
                    'checksum': image.checksum,
                    'cats': list(
                        map(
                            int,
                            list(filter(bool, user_image.image_cat.split(',')))
                        )
                    )
                }
                if image.thumbnail:
                    field_obj['thumbnail'] = image.thumbnail.url
                    field_obj['height'] = image.height
                    field_obj['width'] = image.width
                response['images'].append(field_obj)
    return JsonResponse(
        response,
        status=status
    )


# save changes or create a new category
@login_required
def save_category(request):
    status = 405
    response = {}
    response['entries'] = []
    if request.is_ajax() and request.method == 'POST':
        ids = request.POST.getlist('ids[]')
        titles = request.POST.getlist('titles[]')
        ImageCategory.objects.filter(
            category_owner=request.user
        ).exclude(id__in=ids).delete()
        x = 0
        for the_id in ids:
            the_id = int(the_id)
            the_title = titles[x]
            x += 1
            if 0 == the_id:
                # if the category is new, then create new
                the_cat = ImageCategory(
                    category_title=the_title,
                    category_owner=request.user)
            else:
                # if the category already exists, update the title
                the_cat = ImageCategory.objects.get(pk=the_id)
                the_cat.category_title = the_title
            the_cat.save()
            response['entries'].append(
                {'id': the_cat.id, 'category_title': the_cat.category_title})
        status = 201

    return JsonResponse(
        response,
        status=status
    )



from document.models import AccessRight, Document, DocumentTemplate


class SessionUserInfo():
    """
    Class for string information about users in session
    author: akorovin
    """
    def __init__(self, current_user):
        self.user = current_user
        self.is_owner = False
        self.access_rights = 'read'
        self.document_id = 0
        self.access_rights = dict()

    def create_doc(self, template_id):
        template = DocumentTemplate.objects.filter(
            id=int(template_id)
        ).first()
        if template:
            document = Document.objects.create(
                owner_id=self.user.id,
                template=template
            )
        else:
            document = Document.objects.create(
                owner_id=self.user.id
            )
        return document

    def init_access(self, document_id):
        """
        Initializes access to document by id,
        :param document_id:
        :type document_id:
        :param current_user:
        :type current_user:
        :return: Returns document and bool value that user can access
        :rtype: tuple
        """
        can_access = False
        document = Document.objects.filter(id=int(document_id)).first()
        if document is None:
            return (False, False)
        else:
            self.document_id = document.id
            if document.owner == self.user:
                self.access_rights = 'write'
                self.is_owner = True
                can_access = True
            else:
                self.is_owner = False
                access_rights = AccessRight.objects.filter(
                    document=document,
                    user=self.user
                ).first()
                if access_rights:
                    self.access_rights = access_rights.rights
                    can_access = True
        return (document, can_access)



# Generated by Django 1.11.13 on 2018-08-14 17:33
from django.db import migrations, models
import style.models


class Migration(migrations.Migration):

    replaces = [('style', '0001_initial'), ('style', '0002_auto_20151226_1110')]

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CitationLocale',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.SlugField(help_text='language code of the locale file.', max_length=4)),
                ('contents', models.TextField(help_text='The XML style definiton.')),
            ],
        ),
        migrations.CreateModel(
            name='CitationStyle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The human readable title.', max_length=128)),
                ('short_title', models.SlugField(help_text='A title used for constant names.', max_length=40)),
                ('contents', models.TextField(help_text='The XML style definiton.')),
            ],
        ),
        migrations.CreateModel(
            name='DocumentFont',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The human readable title.', max_length=128)),
                ('font_file', models.FileField(help_text='The font file.', upload_to='document-font')),
                ('fontface_definition', models.TextField(help_text='The CSS definition of the font face (everything inside of @font-face{}). Add [URL] where the link to the font file is to appear.')),
            ],
        ),
        migrations.CreateModel(
            name='DocumentStyle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The human readable title.', max_length=128)),
                ('filename', models.SlugField(help_text='The base of the filenames the style occupies.', max_length=20)),
                ('contents', models.TextField(help_text='The CSS style definiton.')),
                ('fonts', models.ManyToManyField(blank=True, default=None, to='style.DocumentFont')),
            ],
        ),
    ]



from builtins import map
from urllib.request import urlopen
from urllib.parse import urlparse

from django.template.defaultfilters import slugify
from django.core.files.base import ContentFile
from django.dispatch import receiver

from avatar.models import Avatar
from allauth.account.signals import user_signed_up
# This file is split of from django-allauth and is licensed as:

# Copyright (c) 2010 Raymond Penners and contributors

# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


def name_from_url(url):
    """
    >>> name_from_url('http://google.com/dir/file.ext')
    u'file.ext'
    >>> name_from_url('http://google.com/dir/')
    u'dir'
    >>> name_from_url('http://google.com/dir')
    u'dir'
    >>> name_from_url('http://google.com/dir/..')
    u'dir'
    >>> name_from_url('http://google.com/dir/../')
    u'dir'
    >>> name_from_url('http://google.com')
    u'google.com'
    >>> name_from_url('http://google.com/dir/subdir/file..ext')
    u'file.ext'
    """
    p = urlparse(url)
    for base in (p.path.split('/')[-1],
                 p.path,
                 p.netloc):
        name = ".".join([s for s in map(slugify, base.split(".")) if s])
        if name:
            return name


def copy_avatar(request, user, account):
    url = account.get_avatar_url()
    if url:
        ava = Avatar(user=user)
        ava.primary = Avatar.objects.filter(user=user).count() == 0
        try:
            content = urlopen(url).read()
            name = name_from_url(url)
            ava.avatar.save(name, ContentFile(content))
        except IOError:
            # Let's not make a big deal out of this...
            pass


@receiver(user_signed_up)
def on_user_signed_up(sender, request, *args, **kwargs):
    sociallogin = kwargs.get('sociallogin')
    if sociallogin:
        copy_avatar(request,
                    sociallogin.account.user,
                    sociallogin.account)



from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from django.db.models import Q

from document.models import DocumentTemplate
from document.helpers.serializers import PythonWithURLSerializer


@login_required
def list(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        status = 200
        doc_templates = DocumentTemplate.objects.filter(
            Q(user=request.user) | Q(user=None)
        )
        date_format = '%Y-%m-%d'
        response['document_templates'] = [
            {
                'id': obj.id,
                'title': obj.title,
                'is_owner': (obj.user is not None),
                'added': obj.added.strftime(date_format),
                'updated': obj.updated.strftime(date_format)
            } for obj in doc_templates
        ]
    return JsonResponse(
        response,
        status=status
    )


@login_required
def get(request):
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse({}, status=405)
    id = int(request.POST['id'])
    if id == 0:
        doc_template = DocumentTemplate()
        doc_template.user = request.user
        doc_template.save()
        status = 201
    else:
        doc_template = DocumentTemplate.objects.filter(
            id=id,
            user=request.user
        ).first()
        status = 200
    if doc_template is None:
        return JsonResponse({}, status=405)
    serializer = PythonWithURLSerializer()
    export_templates = serializer.serialize(
        doc_template.exporttemplate_set.all()
    )
    document_styles = serializer.serialize(
        doc_template.documentstyle_set.all(),
        use_natural_foreign_keys=True,
        fields=['title', 'slug', 'contents', 'documentstylefile_set']
    )
    response = {
        'template': {
            'id': doc_template.id,
            'title': doc_template.title,
            'definition': doc_template.definition,
            'export_templates': export_templates,
            'document_styles': document_styles
        },
    }
    return JsonResponse(
        response,
        status=status
    )


@login_required
def save(request):
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse({}, status=405)
    id = request.POST['id']
    doc_template = DocumentTemplate.objects.filter(
        id=id,
        user=request.user
    ).first()
    if doc_template is None:
        return JsonResponse({}, status=405)
    response = {}
    status = 200
    doc_template.definition = request.POST['value']
    doc_template.title = request.POST['title']
    doc_template.import_id = request.POST['import_id']
    doc_template.save()
    return JsonResponse(
        response,
        status=status
    )


@login_required
def copy(request):
    if not request.is_ajax() or request.method != 'POST':
        return JsonResponse({}, status=405)
    id = request.POST['id']
    doc_template = DocumentTemplate.objects.filter(
        Q(id=id),
        Q(user=request.user) | Q(user=None)
    ).first()
    if doc_template is None:
        return JsonResponse({}, status=405)
    response = {}
    status = 201
    document_styles = [style for style in doc_template.documentstyle_set.all()]
    export_templates = [
        template for template in doc_template.exporttemplate_set.all()
    ]
    doc_template.pk = None
    doc_template.user = request.user
    doc_template.save()
    for ds in document_styles:
        style_files = [file for file in ds.documentstylefile_set.all()]
        ds.pk = None
        ds.document_template = doc_template
        ds.save()
        for sf in style_files:
            sf.pk = None
            sf.style = ds
            sf.save()
    for et in export_templates:
        et.pk = None
        et.document_template = doc_template
        et.save()
    response['new_id'] = doc_template.id
    return JsonResponse(
        response,
        status=status
    )


@login_required
def delete(request):
    response = {}
    status = 405
    if request.is_ajax() and request.method == 'POST':
        id = int(request.POST['id'])
        doc_template = DocumentTemplate.objects.filter(
            pk=id,
            user=request.user
        ).first()
        if doc_template:
            status = 200
            if doc_template.is_deletable():
                doc_template.delete()
                response['done'] = True
            else:
                response['done'] = False
    return JsonResponse(
        response,
        status=status
    )



# Generated by Django 2.2.4 on 2019-08-31 14:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0018_documenttemplate_import_id'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documenttemplate',
            name='definition_hash',
        ),
    ]
